event      = require("component-event")
serlialize = require("@rill/form-json")
parseURL   = require("@rill/parse-url")
expect     = require("../util/expect")

hijack =
	###
	# Listen to all url change events on a dom element and trigger the server callback.
	#
	# @param {DOMNode} element
	# @api private
	###
	listen: (element = window)->
		event.bind(element, "DOMContentLoaded", handleChange)
		event.bind(element, "popstate", handleChange)
		event.bind(element, "submit", handleClick)
		event.bind(element, "click", handleClick)
		return hijack

	###
	# Add a handler to listen to url changes.
	#
	# @param {Function} handler
	# @api private
	###
	createServer: (handler)->
		createRequest = handler
		return hijack

	###
	# Trigger the registered handler to navigate to a given url.
	#
	# @param {String} url
	# @param {Object} options
	# @api private
	###
	goTo: (url, { method, headers, body, files, title } = {})->
		title ?= ""
		req    =
			url:     url
			body:    body
			files:   files
			method:  method ? "GET"
			headers: headers ? {}


		createRequest(req).then(({ redirected })->
			window.history.pushState(req, title, url) unless redirected
			return
		)

# Store registered createServer callback.
createRequest = ->

###
# Handle an event that changed the url (popstate or page load)
#
# @param {Object} event
###
handleChange = (event)->
	hijack.goTo(window.location.href,
		method: "GET",
		replaceState: true
	)

###
# Handle intercepting form and link clicks to update the url.
#
# @param {Object} event
###
handleClick = (event)->
	# Ignore canceled events, modified clicks, and right clicks.
	return if event.defaultPrevented or
			  event.metaKey  or
			  event.ctrlKey  or
			  event.shiftKey or
			  event.button   isnt 0

	# Get the <a> or <form> element.
	el = event.target
	el = el.parentNode while el? and
							 el.nodeName isnt 'A' and
							 el.nodeName isnt "FORM"

	# Ignore clicks from non-a elements.
	return unless el?

	# Ignore the click if the element has a target.
	return if el.target and el.target isnt "_self"

	# Ignore the click if it's a download link. (We use this method of
	# detecting the presence of the attribute for old IE versions.)
	return if Boolean(el.attributes.download)

	# Use a url parser to parse URLs instead of relying on the browser
	# to do it for us (because IE).
	url = el.href or el.action
	{ protocol, host } = parseURL(url, window.location)

	# Ignore links that don't share a protocol and host with ours.
	return unless window.location.protocol.indexOf(protocol) isnt -1 and
			      window.location.host is host

	# Ignore 'rel="external"' links.
	return if el.rel and /(?:^|\s+)external(?:\s+|$)/.test(el.rel)


	{ body, files } = serlialize(el) if el.nodeName is "FORM"

	hijack.goTo(url,
		method: el.method
		body:   body
		files:  files
	)

	event.preventDefault()

module.exports = hijack
